//
// Class definitions for operator and operator list. These will be used to
// calculate the criticalities of atoms.
//
// O.S. 25/02/2002
//

#ifndef OPERATOR_H
#define OPERATOR_H

class Atom;

class OperatorAtom
{
public:
    OperatorAtom() { next = 0; a = 0; negated = false; };
    ~OperatorAtom() {};

    OperatorAtom *next;
    
    Atom *a;
    bool negated:1;	// True means the atom in the body of the rule(operator)
			// is a not-atom. Reverse of atom's criticality value
			// will be used while calculating criticality of op.
};

class Operator
{
public:
    Operator() { tail = &list; generatenot = false; killingrule = false;
		 choicerule = false; catom_index = -1; };
    ~Operator();

    bool generatenot:1;		// True means the operator(rule) generate a not-
				// atom (i.e. the head of the rule is a not-atom
				// ). Reverse of the rule's criticality value
				// will be used while calculating criticality of
				// head atom.
    bool killingrule:1;		// This is true for _false :- a, b, not c. type
				// model killing rules.
    bool choicerule:1;
    long catom_index;		// Points to the catom of the head in catom_list

    void push(OperatorAtom *opa);
    OperatorAtom *head();

private:
    OperatorAtom list;
    OperatorAtom *tail;
};

class OperatorNode
{
public:
    OperatorNode() { next = 0; op = 0; };
    ~OperatorNode() {};

    OperatorNode *next;
    Operator *op;
};

class OperatorList
{
public:
    OperatorList() { tail = &list; };
    ~OperatorList();

    void push(OperatorNode *opn);
    void push(Operator *op);
    OperatorNode *head();

private:
    OperatorNode list;
    OperatorNode *tail;
};


inline OperatorAtom * Operator::head()
{
    return list.next;
}

inline OperatorNode * OperatorList::head()
{
    return list.next;
}

inline void Operator::push(OperatorAtom *opa)
{
    tail->next = opa;
    tail = opa;
}

inline void OperatorList::push(OperatorNode *opn)
{
    tail->next = opn;
    tail = opn;
}

inline void OperatorList::push(Operator *op)
{
    OperatorNode *opn = new OperatorNode;
    opn->op = op;
    push(opn);
}

#endif
