//
// sort_op function is separated from smodels.cc to make compile-try cycle fast
//

// 02/07/2002 Austin
#include <cmath>

#include <iostream>
#include "atomrule.h"

bool max_total_1(Atom *a1, Atom *a2);
bool max_total_2(Atom *a1, Atom *a2);
bool max_posminusneg(Atom *a1, Atom *a2);
bool sort_efforts(Atom *a1, Atom *a2);
bool sort_efforts_minmax(Atom *a1, Atom *a2);
bool sort_efforts_sum(Atom *a1, Atom *a2);
bool sort_efforts_gm2d(Atom *a1, Atom *a2);
bool sort_efforts_max(Atom *a1, Atom *a2);
bool sort_efforts_product(Atom *a1, Atom *a2);
bool sort_efforts_min(Atom *a1, Atom *a2);
bool sort_efforts_ratio(Atom *a1, Atom *a2);
bool sort_efforts_sumdiff(Atom *a1, Atom *a2);
bool sort_efforts_sumodiff(Atom *a1, Atom *a2);
bool sort_efforts_diff(Atom *a1, Atom *a2);

bool sort_op_hard(Atom *a1, Atom *a2)
{
    return (sort_efforts_sum(a1,a2));
}

bool sort_op(Atom *a1, Atom *a2)
{
    return (sort_efforts(a1,a2));
}

bool max_total_1(Atom *a1, Atom *a2)
{
    double tmp1 = a1->criticality + a1->negcriticality;
    double tmp2 = a2->criticality + a2->negcriticality;

    if ((a1->criticality != 0.5) && (a1->negcriticality != 0.5))
        tmp1 += 1;
    if ((a2->criticality != 0.5) && (a2->negcriticality != 0.5))
        tmp2 += 1;
   
    if (tmp1 == 2.0)
        tmp1 = 0;
    if (tmp2 == 2.0)
        tmp2 = 0;

    return tmp1 < tmp2;
}
bool max_total_2(Atom *a1, Atom *a2)
{
    double tmp1 = a1->criticality + a1->negcriticality;
    double tmp2 = a2->criticality + a2->negcriticality;

    if ((a1->Bpos) || (a1->Bneg))
        tmp1 = 0;
    if ((a2->Bpos) || (a2->Bneg))
        tmp2 = 0;
   
    return tmp1 <= tmp2;
}

bool max_posminusneg(Atom *a1, Atom *a2)
{
    double tmp1 = a1->criticality + (1.0 - a1->negcriticality);
    double tmp2 = a2->criticality + (1.0 - a2->negcriticality);

    if ((a1->Bpos) || (a1->Bneg))
        tmp1 = 0;
    if ((a2->Bpos) || (a2->Bneg))
        tmp2 = 0;

    return tmp1 <= tmp2;
}

bool sort_efforts_diff(Atom *a1, Atom *a2)
{
    if (a1->Bpos || a1->Bneg)
	return false;
    if (a2->Bpos || a2->Bneg)
	return true;

    double diff1 = fabs(a1->posEffort - a1->negEffort);
    double diff2 = fabs(a2->posEffort - a2->negEffort);

    return diff1 > diff2;
}

bool sort_efforts_sumdiff(Atom *a1, Atom *a2)
{
    double diff1 = fabs(a1->posEffort - a1->negEffort);
    double diff2 = fabs(a2->posEffort - a2->negEffort);
    double sum1 = a1->posEffort + a1->negEffort;
    double sum2 = a2->posEffort + a2->negEffort;

    double tmp1 = diff1/sum1;
    double tmp2 = diff2/sum2;

    if (sum1 == 0)
	tmp1 = 1;
    if (sum2 == 0)
	tmp2 = 1;

    return tmp1 > tmp2;
}

bool sort_efforts_sumodiff(Atom *a1, Atom *a2)
{
    double diff1 = fabs(a1->posEffort - a1->negEffort);
    double diff2 = fabs(a2->posEffort - a2->negEffort);
    double sum1 = a1->posEffort + a1->negEffort;
    double sum2 = a2->posEffort + a2->negEffort;

    double tmp1 = sum1/diff1;
    double tmp2 = sum2/diff2;

    if (a1->posEffort == 0 && a1->negEffort == 0)
        tmp1 = 0;
    if (a2->posEffort == 0 && a2->negEffort == 0)
        tmp2 = 0;

    return tmp1 < tmp2;
}

bool sort_efforts_min(Atom *a1, Atom *a2)
{
    //double diff1 = fabs(a1->posEffort - a1->negEffort);
    //double diff2 = fabs(a2->posEffort - a2->negEffort);
    double min1 = (a1->posEffort>a1->negEffort?a1->negEffort:a1->posEffort);
    double min2 = (a2->posEffort>a2->negEffort?a2->negEffort:a2->posEffort);

    return min1 > min2;
}

bool sort_efforts_sum(Atom *a1, Atom *a2)
{
    //double diff1 = fabs(a1->posEffort - a1->negEffort);
    //double diff2 = fabs(a2->posEffort - a2->negEffort);
    double tmp1 = (a1->posEffort + a1->negEffort);
    double tmp2 = (a2->posEffort + a2->negEffort);

    return tmp1 < tmp2;
}

bool sort_efforts_product(Atom *a1, Atom *a2)
{
    double pro1 = (a1->posEffort * a1->negEffort);
    double pro2 = (a2->posEffort * a2->negEffort);

    return pro1 < pro2;
}

bool sort_efforts_gm2d(Atom *a1, Atom *a2)
{
    if (a1->Bpos || a1->Bneg)
	return true;
    if (a2->Bpos || a2->Bneg)
	return false;

    double diff1 = fabs(a1->posEffort - a1->negEffort);
    double diff2 = fabs(a2->posEffort - a2->negEffort);
    double max1 = (a1->posEffort>a1->negEffort?a1->posEffort:a1->negEffort);
    double max2 = (a2->posEffort>a2->negEffort?a2->posEffort:a2->negEffort);

    return max1-(2*diff1) < max2-(2*diff2);
}

bool sort_efforts_max(Atom *a1, Atom *a2)
{
    double max1 = (a1->posEffort>a1->negEffort?a1->posEffort:a1->negEffort);
    double max2 = (a2->posEffort>a2->negEffort?a2->posEffort:a2->negEffort);

    return max1 < max2;
}

bool sort_efforts_ratio(Atom *a1, Atom *a2)
{
    double min1, max1;
    double min2, max2;

    if (a1->negEffort > a1->posEffort)
    {
        min1 = a1->posEffort;
        max1 = a1->negEffort;
    }
    else
    {
        min1 = a1->negEffort;
        max1 = a1->posEffort;
    }

    if (a2->negEffort > a2->posEffort)
    {
        min2 = a2->posEffort;
        max2 = a2->negEffort;
    }
    else
    {
        min2 = a2->negEffort;
        max2 = a2->posEffort;
    }

    double tmp1 = min1 * (min1 / (min1+max1));
    double tmp2 = min2 * (min2 / (min2+max2));

    if ((min1+max1) == 0)
    	tmp1 = 0;
    if ((min2+max2) == 0)
    	tmp2 = 0;
  
    return tmp1 < tmp2; 
}

bool sort_efforts_minmax(Atom *a1, Atom *a2)
{
    double min1, max1;
    double min2, max2;

    if (a1->negEffort > a1->posEffort)
    {
        min1 = a1->posEffort;
        max1 = a1->negEffort;
    }
    else
    {
        min1 = a1->negEffort;
        max1 = a1->posEffort;
    }

    if (a2->negEffort > a2->posEffort)
    {
        min2 = a2->posEffort;
        max2 = a2->negEffort;
    }
    else
    {
        min2 = a2->negEffort;
        max2 = a2->posEffort;
    }

    if ((max1<max2) || ((max1==max2) && (min1<min2)))
        return false;
    else
        return true;
}

bool sort_efforts(Atom *a1, Atom *a2)
{
    double min1, max1;
    double min2, max2;

    if (a1->negEffort > a1->posEffort)
    {
	min1 = a1->posEffort;
	max1 = a1->negEffort;
    }
    else
    {
	min1 = a1->negEffort;
	max1 = a1->posEffort;
    }

    if (a2->negEffort > a2->posEffort)
    {
	min2 = a2->posEffort;
	max2 = a2->negEffort;
    }
    else
    {
	min2 = a2->negEffort;
	max2 = a2->posEffort;
    }

    if ((min1>min2) || ((min1==min2) && (max1>max2)))
	return false;
    else
	return true;
}

